package com.jkcredit.location.util;

import com.jkcredit.location.enums.DigestALGEnum;
import org.apache.commons.codec.binary.Hex;

import java.security.MessageDigest;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * 签名工具
 * create by Mr.Liu
 */
public final class ApiDigestUtil {


	/**
	 * 对 dataMap 的内容进行签名，并返回签名值
	 *
	 * @param dataMap          待签名kv
	 * @param securityCheckKey 安全密钥
	 * @param de               签名算法
	 * @param encoding         签名编码名称
	 */
	public static String execute(Map<String, String> dataMap, String securityCheckKey, DigestALGEnum de, String encoding) {
		if (dataMap == null) {
			throw new RuntimeException("数据不能为空");
		}
		if (securityCheckKey == null || "".equals(securityCheckKey)) {
			throw new RuntimeException("安全校验码不能为空");
		}
		if (de == null) {
			throw new RuntimeException("摘要算法不能为空");
		}
		if (encoding == null || "".equals(encoding)) {
			throw new RuntimeException("字符集不能为空");
		}

		String message = sort(dataMap) + securityCheckKey;

		/**执行签名**/
		byte[] toDigest;
		try {
			toDigest = message.getBytes(encoding);
			MessageDigest md = MessageDigest.getInstance(de.getCode());
			md.update(toDigest);
			return new String (Hex.encodeHex(md.digest()));
		} catch (Exception e) {
			throw new RuntimeException("签名失败", e);
		}
	}

	/**
	 * 排序字符串
	 * @param paramMap
	 * @return
	 */
	public static String sort(Map<String, String> paramMap) {
		/**按a~z、A~Z排序**/
		TreeMap<String, String> treeMap = new TreeMap<String, String>(paramMap);

		/**拼接签名字符串**/
		StringBuilder sb = new StringBuilder();
		for (Map.Entry<String, String> entry : treeMap.entrySet()) {
			if ("sign".equals(entry.getKey())) {
				continue;
			}
			sb.append(entry.getKey()).append("=").append(defaultIfBlank(entry.getValue(), "")).append("&");
		}

		/**整理字符串**/
		if (sb.length() > 0) {
			sb.setLength(sb.length() - 1);
		}
		return sb.toString();
	}

	/**
	 * 为空默认
	 * @param value
	 * @param defaultValue
	 * @return
	 */
	private static String defaultIfBlank(String value, String defaultValue) {
		if (value == null || "".equals(value)) {
			return defaultValue;
		} else {
			return value;
		}
	}

	/**
	 * 将 Map<String, String[]> -> Map<String, String>
	 *
	 * @param paramMap
	 * @return 转换完成的集合
	 */
	public static Map<String, String> toMap(Map<String, String[]> paramMap){
		Map<String, String> returnMap = new HashMap<String, String>();
		if (paramMap != null) {
			for (Map.Entry<String, String[]> entry : paramMap.entrySet()) {
				returnMap.put(entry.getKey(), entry.getValue()[0]);
			}
		}
		return returnMap;

	}

	/**
	 * 清除空元素
	 * @param paramMap
	 */
	private static void clearNullValue(Map<String, String> paramMap) {
		Iterator<Map.Entry<String, String>> it = paramMap.entrySet().iterator();
		while(it.hasNext()){
			Map.Entry<String, String> entry = it.next();
			if ((entry.getValue()) == null || "".equals((entry.getValue()))) {
				it.remove();
			}
		}
	}
}
